@@ -6,10 +6,11 @@ See the specification docs and the JSON Schema file for more information.
 """
 
 import json
+import math
 import warnings
 from collections import defaultdict
 from datetime import datetime
-from typing import List, Union, Optional, Dict, ClassVar, cast
+from typing import List, Union, Optional, Dict, cast
 
 import jsonschema.validators
 
@@ -31,9 +32,6 @@ class Mmif(MmifObject):
     :param validate: whether to validate the data against the MMIF JSON schema.
     """
 
-    view_prefix: ClassVar[str] = 'v_'
-    id_delimiter: ClassVar[str] = ':'
-
     def __init__(self, mmif_obj: Optional[Union[bytes, str, dict]] = None, *, validate: bool = True) -> None:
         self.metadata: MmifMetadata = MmifMetadata()
         self.documents: DocumentsList = DocumentsList()
@@ -93,26 +91,50 @@ class Mmif(MmifObject):
     def _deserialize(self, input_dict: dict) -> None:
         """
         Deserializes the MMIF JSON string into a Mmif object.
-        This will read in existing ``Annotation`` typed annotations and 
+        After *regular* deserialization, this method will perform the following 
+        *special* handling of Annotation.properties that allows apps to access 
+        Annotation/Document properties that are not encoded in the objects 
+        themselves. This is to allow apps to access in a more intuitive way, 
+        without having too much hassle to iterate views and manually collect the properties.
+        
+        1. This will read in existing *view*-scoped properties from *contains*
+        metadata and attach them to the corresponding ``Annotation`` objects.
+
+        1. This will read in existing ``Annotation`` typed annotations and 
         attach the document-level properties to the ``Document`` objects, 
-        using a volatile property dict. This will allow apps to access the
-        document-level properties without having too much hassle to iterate
-        views and manually collect the properties.
+        using an ephemeral property dict. 
+        
         """
         super()._deserialize(input_dict)
         for view in self.views:
-            doc_id = None
-            if AnnotationTypes.Annotation in view.metadata.contains:
-                if 'document' in view.metadata.contains[AnnotationTypes.Annotation]:
-                    doc_id = view.metadata.contains[AnnotationTypes.Annotation]['document']
+            # this dict will be populated with properties 
+            # that are not encoded in individual annotations objects themselves
+            extrinsic_props = defaultdict(dict)
+            for at_type, type_lv_props in view.metadata.contains.items():
+                for prop_key, prop_value in type_lv_props.items():
+                    extrinsic_props[at_type][prop_key] = prop_value
+            for ann in view.get_annotations():
+                # first add all extrinsic properties to the Annotation objects
+                # as "ephemeral" properties
+                for prop_key, prop_value in extrinsic_props[ann.at_type].items():
+                    ann._props_ephemeral[prop_key] = prop_value
+                # then, do the same to associated Document objects. Note that, 
                 # in a view, it is guaranteed that all Annotation objects are not duplicates
-                for ann in view.get_annotations(AnnotationTypes.Annotation):
-                    if doc_id is None:
-                        doc_id = ann.get_property('document')
+                if ann.at_type == AnnotationTypes.Annotation:
+                    doc_id = ann.get_property('document')
                     try:
-                        self.get_document_by_id(doc_id)._add_property_from_annotation(ann)
+                        for prop_key, prop_value in ann.properties.items():
+                            self.get_document_by_id(doc_id)._props_ephemeral[prop_key] = prop_value
                     except KeyError:
-                        warnings.warn(f"Annotation {ann.id} has a document ID {doc_id} that does not exist in the MMIF object. Skipping.", RuntimeWarning)
+                        warnings.warn(f"Annotation {ann.id} (in view {view.id}) has a document ID {doc_id} that "
+                                      f"does not exist in the MMIF object. Skipping.", RuntimeWarning)
+                # lastly, add quick access to `start` and `end` values if the annotation is using `targets` property
+                if 'targets' in ann.properties:
+                    if 'start' in ann.properties or 'end' in ann.properties:
+                        raise ValueError(f"Annotation {ann.id} (in view {view.id}) has `targes` and `start`/`end/` "
+                                         f"properties at the same time. Annotation anchors are ambiguous.")
+                    ann._props_ephemeral['start'] = self._get_linear_anchor_point(ann, start=True)
+                    ann._props_ephemeral['end'] = self._get_linear_anchor_point(ann, start=False)
 
     def generate_capital_annotations(self):
         """
@@ -143,9 +165,9 @@ class Mmif(MmifObject):
                             doc_id = ann.get_property('document')
                         existing_anns[doc_id].update(ann.properties)
                 for doc in view.get_documents():
-                    anns_to_write[doc.id].update(doc._props_temporary)
+                    anns_to_write[doc.id].update(doc._props_pending)
             for doc in self.documents:
-                anns_to_write[doc.id].update(doc._props_temporary)
+                anns_to_write[doc.id].update(doc._props_pending)
             # additional iteration of views, to find a proper view to add the 
             # generated annotations. If none found, use the last view as the kitchen sink
             last_view_for_docs = defaultdict(lambda: last_view)
@@ -337,8 +359,8 @@ class Mmif(MmifObject):
         :return: a reference to the corresponding document, if it exists
         :raises KeyError: if there is no corresponding document
         """
-        if Mmif.id_delimiter in doc_id:
-            vid, did = doc_id.split(Mmif.id_delimiter)
+        if self.id_delimiter in doc_id:
+            vid, did = doc_id.split(self.id_delimiter)
             view = self[vid]
             if isinstance(view, View):
                 return view.get_document_by_id(did) 
@@ -385,8 +407,8 @@ class Mmif(MmifObject):
                     aligned_types = set()
                     for ann_id in [alignment['target'], alignment['source']]:
                         ann_id = cast(str, ann_id)
-                        if Mmif.id_delimiter in ann_id:
-                            view_id, ann_id = ann_id.split(Mmif.id_delimiter)
+                        if self.id_delimiter in ann_id:
+                            view_id, ann_id = ann_id.split(self.id_delimiter)
                             aligned_type = cast(Annotation, self[view_id][ann_id]).at_type
                         else:
                             aligned_type = cast(Annotation, alignment_view[ann_id]).at_type
@@ -411,9 +433,10 @@ class Mmif(MmifObject):
                 next(annotations)
                 views.append(view)
             except StopIteration:
-                # search failed by the full doc_id string, now try trimming the view_id from the string and re-do the search
-                if Mmif.id_delimiter in doc_id:
-                    vid, did = doc_id.split(Mmif.id_delimiter)
+                # means search failed by the full doc_id string, 
+                # now try trimming the view_id from the string and re-do the search
+                if self.id_delimiter in doc_id:
+                    vid, did = doc_id.split(self.id_delimiter)
                     if view.id == vid:
                         annotations = view.get_annotations(document=did)
                         try:
@@ -462,6 +485,54 @@ class Mmif(MmifObject):
                 if at_types in view.metadata.contains:
                     return view
         return None
+    
+    def _get_linear_anchor_point(self, ann: Annotation, targets_sorted=False, start: bool = True) -> Union[int, float]:
+        # TODO (krim @ 2/5/24): Update the return type once timeunits are unified to `ms` as integers (https://github.com/clamsproject/mmif/issues/192)
+        """
+        Retrieves the anchor point of the annotation. Currently, this method only supports linear anchors, 
+        namely time and text, hence does not work with spatial anchors (polygons or video-object).
+        
+        :param ann: An Annotation object that has a linear anchor point. Namely, some subtypes of `Region` vocabulary type.
+        :param start: If True, returns the start anchor point. Otherwise, returns the end anchor point. N/A for `timePoint` anchors.
+        :param targets_sorted: If True, the method will assume that the targets are sorted in the order of the anchor points.
+        :return: the anchor point of the annotation. 1d for linear regions (time, text)
+        """
+        props = ann.properties
+        if 'timePoint' in props:
+            return ann.get_property('timePoint')
+        elif 'targets' in props:
+            
+            def get_target_ann(cur_ann, target_id):
+                if self.id_delimiter not in target_id:
+                    target_id = self.id_delimiter.join((cur_ann.parent, target_id))
+                return self.__getitem__(target_id)
+            
+            if not targets_sorted:
+                point = math.inf if start else -1
+                comp = min if start else max
+                for target_id in ann.get_property('targets'):
+                    target = get_target_ann(ann, target_id)
+                    point = comp(point, self._get_linear_anchor_point(target, start=start))
+                return point
+            target_id = ann.get_property('targets')[0 if start else -1]
+            target = get_target_ann(ann, target_id)
+            return self._get_linear_anchor_point(target, start=start)
+        elif (start and 'start' in props) or (not start and 'end' in props):
+            return ann.get_property('start' if start else 'end')
+        else:
+            raise ValueError(f"{ann.id} ({ann.at_type}) does not have a valid anchor point. Is it a valid 'Region' type?")
+    
+    def get_start(self, annotation: Annotation) -> Union[int, float]:
+        """
+        An alias to `get_anchor_point` method with `start=True`.
+        """
+        return self._get_linear_anchor_point(annotation, start=True)
+    
+    def get_end(self, annotation: Annotation) -> Union[int, float]:
+        """
+        An alias to `get_anchor_point` method with `start=False`.
+        """
+        return self._get_linear_anchor_point(annotation, start=False)
 
     # pytype: disable=bad-return-type
     def __getitem__(self, item: str) -> Union[Document, View, Annotation]:
@@ -475,7 +546,7 @@ class Mmif(MmifObject):
         """
         if item in self._named_attributes():
             return self.__dict__[item]
-        split_attempt = item.split(Mmif.id_delimiter)
+        split_attempt = item.split(self.id_delimiter)
 
         document_result = self.documents.get(split_attempt[0])
         view_result = self.views.get(split_attempt[0])
