@@ -6,4 +6,4 @@ from .base_types import DocumentTypesBase
 from .annotation_types import AnnotationTypes
 from .document_types import DocumentTypes
 
-typevers = {**ThingType.typevers, **AnnotationTypes.typevers, **DocumentTypes.typevers}
+_typevers = {**ThingType._typevers, **AnnotationTypes._typevers, **DocumentTypes._typevers}
